package com.zhongshu.reward.client.model.param;

import com.zhongshu.reward.client.type.TimeUnitType;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import org.checkerframework.checker.units.qual.min;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;

/**
 * @author wjf
 * @date 2024/8/12
 */
@Data
public class TransferRulerParam {

    private String id;

    /**
     * 单笔最小提现金额
     */
    @ApiModelProperty("单笔最小提现金额")
    @Max(value = 20000, message = "单笔提现金额不得超过 200")
    @Min(value = 30, message = "单笔提现金额不得低于 0.3")
    private BigDecimal minTotal = BigDecimal.ZERO;

    /**
     * 单笔最大提现金额
     */
    @ApiModelProperty("单笔最大提现金额")
    @Max(value = 20000, message = "单笔提现金额不得超过 200")
    @Min(value = 30, message = "单笔提现金额不得低于0.3")
    private BigDecimal maxTotal = BigDecimal.ZERO;

    /**
     * 单日最大转账金额
     */
    @ApiModelProperty("单日最大转账金额")
    @Max(value = 2000000, message = "单日最大转账金额不得超过20000")
    private BigDecimal dayMaxTotal = BigDecimal.ZERO;

    /**
     * 提现手续费
     */
    @ApiModelProperty("提现手续费")
    private BigDecimal commission = BigDecimal.ZERO;

    /**
     * 提现次数
     */
    @ApiModelProperty("提现次数")
    private Integer size;

    /**
     * 时间单位
     */
    @ApiModelProperty("时间单位")
    private TimeUnitType timeUnit;

    /**
     * 每月开始号数
     */
    @ApiModelProperty("每月开始号数")
    @Max(value = 31, message = "超出月份天数")
    @Min(value = 1, message = "日数最小为1")
    private Integer startDay;

    /**
     * 每月结束号数
     */
    @ApiModelProperty("每月结束号数")
    @Max(value = 31, message = "超出月份天数")
    @Min(value = 1, message = "日数最小为1")
    private Integer endDay;

    /**
     * 每日开始时
     */
    @ApiModelProperty("每日开始时")
    @Max(value = 24, message = "不过超过23时")
    @Min(value = 0, message = "不得小于0时")
    private Integer startHour;

    /**
     * 每日开始分
     */
    @Max(value = 59, message = "不过超过59分")
    @Min(value = 0, message = "不得小于0分")
    @ApiModelProperty("每日开始分")
    private Integer startMinute;

    /**
     * 每日结束时
     */
    @Max(value = 24, message = "不过超过24时")
    @Min(value = 0, message = "不得小于0时")
    @ApiModelProperty("每日结束时")
    private Integer endHour;

    /**
     * 每日结束分
     */
    @Max(value = 59, message = "不过超过59分")
    @Min(value = 0, message = "不得小于0分")
    @ApiModelProperty("每日结束分")
    private Integer endMinute;
}
