package com.zhongshu.reward.server.core.service;

import com.github.microservice.productcenter.client.model.ProductRuleModel;
import com.github.microservice.productcenter.client.ret.ResultContent;
import com.github.microservice.productcenter.client.ret.ResultState;
import com.github.microservice.productcenter.client.service.ProductRuleService;
import com.zhongshu.reward.client.type.ReceiptsStatus;
import com.zhongshu.reward.client.type.ReceiptsType;
import com.zhongshu.reward.server.core.dao.WalletDao;
import com.zhongshu.reward.server.core.dao.WalletReceiptsDao;
import com.zhongshu.reward.server.core.domain.Wallet;
import com.zhongshu.reward.server.core.domain.WalletReceipts;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author wjf
 * @date 2024/8/23
 */
@Service
@Slf4j
public class GoodsReceiptsService {

    @Autowired
    WalletDao walletDao;

    @Autowired
    WalletService walletService;

    @Autowired
    WalletReceiptsDao walletReceiptsDao;

    @Autowired
    ProductRuleService productRuleService;

    /**
     * 购物返利 todo// 1、获取产品库 返利规则 2、获取订单详情、获取商品详情
     * @param userId 用户id
     * @param goodsTotal 商品金额
     * @param orderNo 订单号
     * @param goodsId 商品id
     */
    @Transactional
    public void receipts(String userId, BigDecimal goodsTotal, String orderNo, String goodsId){

        ProductRuleModel productRule = getRule(goodsId);
        if (null == productRule){
            return;
        }

        BigDecimal total = getReceiptsTotal(goodsTotal, productRule);

        Wallet wallet = walletService.getWalletByUserId(userId);

        WalletReceipts walletReceipts = new WalletReceipts();
        walletReceipts.setWallet(wallet);
        walletReceipts.setReceiptsType(ReceiptsType.REBATE);
        walletReceipts.setEstimatedTime(new Date().getTime() + 2*24*60*60*1000);
        walletReceipts.setTotal(total);
        walletReceipts.setOutTradeNo(orderNo);
        walletReceipts.setGoodsId(goodsId);
        walletReceipts.setStatus(ReceiptsStatus.WAIT);

        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(wallet.getId(), total);
        log.info("购物返利入账");
    }

    @NotNull
    private static BigDecimal getReceiptsTotal(BigDecimal goodsTotal, ProductRuleModel productRule) {
        BigDecimal total = BigDecimal.ZERO;

        if (productRule.getCashBackFixed()!=null){
            total = productRule.getCashBackFixed();
        }else if (productRule.getCashBackPercent()!=null){
            total = goodsTotal.multiply(productRule.getCashBackPercent().multiply(BigDecimal.TEN.pow(2)));
        } return total;
    }

    @Nullable
    private ProductRuleModel getRule(String goodsId) {
        ResultContent<ProductRuleModel> goodsRulerResult = productRuleService.list(goodsId);
        ResultContent<ProductRuleModel> overRulerResult = productRuleService.list(null);

        ProductRuleModel productRule = goodsRulerResult.getContent();
        ProductRuleModel overRule = overRulerResult.getContent();
        if (productRule.getEnableCashback()!=null && productRule.getEnableCashback()){
            return productRule;
        }else if (overRule.getEnableCashback()!=null && overRule.getEnableCashback()){
            return overRule;
        }
        return null;
    }


    /**
     * 取消购物返利
     * @param userId 用户id
     * @param orderNo 订单号
     * @param goodsIds 商品id
     */
    @Transactional
    public void cancel(String userId, String orderNo, List<String> goodsIds){
        BigDecimal total = BigDecimal.ZERO;
        Wallet wallet = walletService.getWalletByUserId(userId);
        List<WalletReceipts> walletReceiptsList = walletReceiptsDao.findByOutTradeNoAndGoodsIdInAndStatus(orderNo, goodsIds, ReceiptsStatus.WAIT);
        for (WalletReceipts it : walletReceiptsList){
            total = total.add(it.getTotal());
            it.setStatus(ReceiptsStatus.CANCEL);
        }
        walletReceiptsDao.saveAll(walletReceiptsList);
        walletDao.updateIncWaitAmount(wallet.getId(), total.negate());
    }

    /**
     * 入账结算
     */
    @Transactional
    public void settle(){
        //todo 查询到期入账的订单
        WalletReceipts walletReceipts = walletReceiptsDao.findTop1ById("");

        walletReceipts.setReceiptsTime(new Date().getTime());
        walletReceipts.setStatus(ReceiptsStatus.RECEIPTS);
        walletReceiptsDao.save(walletReceipts);
        walletDao.updateIncWaitAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal().negate());
        walletDao.updateIncAmount(walletReceipts.getWallet().getId(), walletReceipts.getTotal());
    }
}
